@@ -1,7 +1,7 @@
 #' Begin A Greedy Pair Switching Search
 #' 
 #' This method creates an object of type greedy_experimental_design and will immediately initiate
-#' a search through $1_{T}$ space for forced balance designs. For debugging, you can use set the \code{seed}
+#' a search through allocation space for forced balance designs. For debugging, you can use set the \code{seed}
 #' parameter and \code{num_cores = 1} to be assured of deterministic output.
 #' 
 #' @param X					The design matrix with $n$ rows (one for each subject) and $p$ columns 
@@ -33,22 +33,27 @@
 #' @param num_cores 		The number of CPU cores you wish to use during the search. The default is \code{1}.
 #' @param seed				The set to set for deterministic output. This should only be set if \code{num_cores = 1} otherwise
 #' 							the output will not be deterministic. Default is \code{NULL} for no seed set.
+#' @param verbose			Should the algorithm emit progress output? Default is \code{TRUE}.
+#' @param use_safe_inverse	Should a regularized inverse be used for the Mahalanobis objective?
+#' 							Default is \code{FALSE}.
 #' @return					An object of type \code{greedy_experimental_design_search} which can be further operated upon
 #' 
 #' @author Adam Kapelner
 #' @examples
-#'  \dontrun{
-#' 	library(MASS)
-#' 	data(Boston)
-#'  #pretend the Boston data was an experiment setting 
-#' 	#first pull out the covariates
-#'  X = Boston[, 1 : 13] 
-#'  #begin the greedy design search
-#' 	ged = initGreedyExperimentalDesignObject(X, 
-#' 		max_designs = 1000, num_cores = 3, objective = "abs_sum_diff")
-#' 	#wait
-#' 	ged
-#' 	}
+#' \dontrun{
+#' set.seed(1)
+#' X = matrix(rnorm(20), nrow = 10)
+#' ged = initGreedyExperimentalDesignObject(
+#'   X,
+#'   max_designs = 5,
+#'   num_cores = 1,
+#'   objective = "abs_sum_diff",
+#'   start = TRUE,
+#'   wait = TRUE,
+#'   verbose = FALSE
+#' )
+#' ged
+#' }
 #' @export
 initGreedyExperimentalDesignObject = function(
 		X = NULL, 
@@ -63,7 +68,9 @@ initGreedyExperimentalDesignObject = function(
 		semigreedy = FALSE, 
 		diagnostics = FALSE,
 		num_cores = 1,
-		seed = NULL){
+		seed = NULL,
+		verbose = TRUE,
+		use_safe_inverse = FALSE){
 	
 	if (!is.null(Kgram)){
 		n = nrow(Kgram)
@@ -78,6 +85,8 @@ initGreedyExperimentalDesignObject = function(
 	verify_objective_function(objective, Kgram, n)
 	
 	checkCount(nT, positive = TRUE, null.ok = FALSE)
+	assertLogical(verbose)
+	assertLogical(use_safe_inverse)
 	
 	if (nT != n / 2 & !is.null(indicies_pairs)){
 		stop("indicies_pairs cannot be specified if nT is not half of n")
@@ -94,7 +103,11 @@ initGreedyExperimentalDesignObject = function(
 	}
 	if (objective == "mahal_dist"){
 		if (p < n){
-			SinvX = solve(var(X))
+			if (use_safe_inverse){
+				SinvX = safe_cov_inverse(X)
+			} else {
+				SinvX = solve(stats::var(X))
+			}
 		}
 	}
 	
@@ -108,6 +121,7 @@ initGreedyExperimentalDesignObject = function(
 	
 	#now go ahead and create the Java object and set its information
 	java_obj = .jnew("GreedyExperimentalDesign.GreedyExperimentalDesign")
+	set_verbose_if_available(java_obj, verbose)
 	.jcall(java_obj, "V", "setMaxDesigns", as.integer(max_designs))
 	.jcall(java_obj, "V", "setNumCores", as.integer(num_cores))	
 	if (!is.null(seed)){
@@ -177,6 +191,7 @@ initGreedyExperimentalDesignObject = function(
 	greedy_experimental_design_search$start = start
 	greedy_experimental_design_search$wait = wait
 	greedy_experimental_design_search$diagnostics = diagnostics
+	greedy_experimental_design_search$verbose = verbose
 	greedy_experimental_design_search$X = X
 	greedy_experimental_design_search$n = n
 	greedy_experimental_design_search$p = p
@@ -233,18 +248,20 @@ summary.greedy_experimental_design_search = function(object, ...){
 #' @method plot greedy_experimental_design_search
 #' @export
 plot.greedy_experimental_design_search = function(x, ...){
-	oldpar <- par(no.readonly = TRUE)    # code line i
-	on.exit(par(oldpar))            # code line i + 1
-	
-	par(mfrow = c(1, 2))
-	
 	progress = greedySearchCurrentProgress(x)
 	res = resultsGreedySearch(x, max_vectors = 2)
-	hist(res$obj_vals_orig_order, br = progress / 10, xlab = "objective value", ylab = NULL, main = paste("After", progress, "searches"))
+	breaks = max(1, round(progress / 10))
+	hist_data = data.frame(obj_val = res$obj_vals_orig_order)
+	hist_plot = ggplot2::ggplot(hist_data, ggplot2::aes(x = obj_val)) +
+		ggplot2::geom_histogram(bins = breaks, color = "black", fill = "grey70") +
+		ggplot2::labs(x = "objective value", y = NULL, title = paste("After", progress, "searches")) +
+		ggplot2::theme_classic()
 #	hist(res$num_iters, br = progress / 10, xlab = "# of search iterations", ylab = NULL, main = "")
 	
 	#now do the plot of number of searches needed
-	plot_obj_val_order_statistic(x)
+	order_plot = build_obj_val_order_statistic_plot(x)
+	draw_plot_pair(hist_plot, order_plot$plot)
+	invisible(list(val_order_stats = order_plot$val_order_stats))
 }
 
 #' Plots the objective value by iteration
@@ -253,6 +270,23 @@ plot.greedy_experimental_design_search = function(x, ...){
 #' @param runs 		A vector of run indices you would like to see plotted (default is to plot the first up to 9)
 #' 
 #' @author 			Adam Kapelner
+#' @examples
+#' \dontrun{
+#' set.seed(1)
+#' X = matrix(rnorm(20), nrow = 10)
+#' ged = initGreedyExperimentalDesignObject(
+#'   X,
+#'   max_designs = 5,
+#'   num_cores = 1,
+#'   diagnostics = TRUE,
+#'   objective = "abs_sum_diff",
+#'   start = TRUE,
+#'   wait = TRUE,
+#'   verbose = FALSE
+#' )
+#' res = resultsGreedySearch(ged, max_vectors = 2)
+#' plot_obj_val_by_iter(res)
+#' }
 #' @export
 plot_obj_val_by_iter = function(res, runs = NULL){
 	if (is.null(res$obj_val_by_iters)){
@@ -264,14 +298,32 @@ plot_obj_val_by_iter = function(res, runs = NULL){
 	}
 	num_to_plot = length(runs)
 	
-	oldpar <- par(no.readonly = TRUE)    # code line i
-	on.exit(par(oldpar))            # code line i + 1
-	
-	par(mfrow = c(ceiling(sqrt(num_to_plot)), ceiling(sqrt(num_to_plot))))
+	plot_data = data.frame(run = character(0), iteration = integer(0), obj_val = numeric(0))
 	for (run in runs){
 		obj_vals = res$obj_val_by_iters[[run]]
-		plot(1 : length(obj_vals), obj_vals, main = paste("Run #", run), xlab = "iteration", ylab = "objective value", type = "o")
+		if (length(obj_vals) == 0){
+			next
+		}
+		plot_data = rbind(plot_data, data.frame(
+			run = paste("Run #", run),
+			iteration = seq_along(obj_vals),
+			obj_val = obj_vals
+		))
 	}
+	plot_data$run = factor(plot_data$run, levels = paste("Run #", runs))
+	if (nrow(plot_data) == 0){
+		blank_plot = build_blank_plot("iteration", "objective value", title = NULL)
+		print(blank_plot)
+		return(invisible(NULL))
+	}
+	plot_data$line_ok = ave(plot_data$iteration, plot_data$run, FUN = function(x) length(x) > 1)
+	iter_plot = ggplot2::ggplot(plot_data, ggplot2::aes(x = iteration, y = obj_val)) +
+		ggplot2::geom_line(data = plot_data[plot_data$line_ok, , drop = FALSE]) +
+		ggplot2::geom_point() +
+		ggplot2::facet_wrap(~run, ncol = ceiling(sqrt(num_to_plot))) +
+		ggplot2::labs(x = "iteration", y = "objective value") +
+		ggplot2::theme_classic()
+	print(iter_plot)
 } 
 
 #' Plots an order statistic of the object value as a function of number of searches
@@ -284,22 +336,141 @@ plot_obj_val_by_iter = function(res, runs = NULL){
 #' @return 				An array of order statistics as a list element
 #' 
 #' @author 				Adam Kapelner
+#' @examples
+#' \dontrun{
+#' set.seed(1)
+#' X = matrix(rnorm(20), nrow = 10)
+#' ged = initGreedyExperimentalDesignObject(
+#'   X,
+#'   max_designs = 5,
+#'   num_cores = 1,
+#'   objective = "abs_sum_diff",
+#'   start = TRUE,
+#'   wait = TRUE,
+#'   verbose = FALSE
+#' )
+#' plot_obj_val_order_statistic(ged, order_stat = 1, skip_every = 1)
+#' }
 #' @export
 plot_obj_val_order_statistic = function(obj, order_stat = 1, skip_every = 5, type = "o", ...){
+	order_plot = build_obj_val_order_statistic_plot(obj, order_stat = order_stat, skip_every = skip_every, type = type, ...)
+	print(order_plot$plot)
+	invisible(list(val_order_stats = order_plot$val_order_stats))	
+}
+
+build_blank_plot = function(xlab, ylab, title = NULL, xlim = NULL, ylim = NULL){
+	blank_data = data.frame(x = 0, y = 0)
+	blank_plot = ggplot2::ggplot(blank_data, ggplot2::aes(x = x, y = y)) +
+		ggplot2::geom_blank() +
+		ggplot2::labs(x = xlab, y = ylab, title = title) +
+		ggplot2::theme_classic()
+	if (!is.null(xlim) || !is.null(ylim)){
+		blank_plot = blank_plot + ggplot2::coord_cartesian(xlim = xlim, ylim = ylim)
+	}
+	blank_plot
+}
+
+draw_plot_pair = function(left_plot, right_plot){
+	grid::grid.newpage()
+	grid::pushViewport(grid::viewport(layout = grid::grid.layout(1, 2)))
+	print(left_plot, vp = grid::viewport(layout.pos.row = 1, layout.pos.col = 1))
+	print(right_plot, vp = grid::viewport(layout.pos.row = 1, layout.pos.col = 2))
+}
+
+build_obj_val_order_statistic_plot = function(obj, order_stat = 1, skip_every = 5, type = "o", ...){
 	progress = greedySearchCurrentProgress(obj)
-	res = resultsGreedySearch(obj, max_vectors = 0)	#don't need any vectors => set it to 0 for speed concerns
+	ylab = paste("objective value (", order_stat, ")", sep = "")
+	dots = list(...)
+	title = NULL
+	if (!is.null(dots$main)){
+		title = dots$main
+	}
+	xlim = dots$xlim
+	ylim = dots$ylim
+	plot_color = NULL
+	if (!is.null(dots$col)){
+		plot_color = dots$col
+	} else if (!is.null(dots$color)){
+		plot_color = dots$color
+	}
+	line_size = dots$lwd
+	point_shape = dots$pch
+	point_size = NULL
+	if (!is.null(dots$cex)){
+		point_size = dots$cex * 1.5
+	}
+	if (progress <= 0 || progress < order_stat){
+		blank_plot = build_blank_plot("Number of Searches", ylab, title = title, xlim = xlim, ylim = ylim)
+		return(list(plot = blank_plot, val_order_stats = numeric(0)))
+	}
+	res = resultsGreedySearch(obj, max_vectors = 1)	#don't need many vectors; keep minimal for speed concerns
 	vals = res$obj_vals_orig_order
 	val_order_stats = array(NA, progress)
-	for (d in order_stat : progress){
-		if (d %% skip_every == 0){
-			val_order_stats[d] = ifelse(order_stat == 1, min(vals[1 : d]), sort(vals[1 : d])[order_stat])
-		}		
+	skip_every = max(1, skip_every)
+	indices = seq(order_stat, progress, by = skip_every)
+	if (length(indices) == 0){
+		indices = progress
+	} else if (tail(indices, 1) != progress){
+		indices = c(indices, progress)
 	}
-	plot(1 : progress, val_order_stats, 
-			xlab = "Number of Searches", 
-			ylab = paste("objective value (", order_stat, ")", sep = ""), 
-			type = type, ...)
-	invisible(list(val_order_stats = val_order_stats))	
+	for (d in indices){
+		vals_d = vals[1 : d]
+		vals_d = vals_d[is.finite(vals_d)]
+		if (length(vals_d) == 0){
+			next
+		}
+		if (order_stat == 1){
+			val_order_stats[d] = min(vals_d)
+		} else if (order_stat <= length(vals_d)){
+			val_order_stats[d] = sort(vals_d)[order_stat]
+		}
+	}
+	if (!any(is.finite(val_order_stats))){
+		blank_plot = build_blank_plot("Number of Searches", ylab, title = title, xlim = xlim, ylim = ylim)
+		return(list(plot = blank_plot, val_order_stats = val_order_stats))
+	}
+	plot_data = data.frame(searches = seq_len(progress), value = val_order_stats)
+	plot_data = plot_data[is.finite(plot_data$value), , drop = FALSE]
+	order_plot = ggplot2::ggplot(plot_data, ggplot2::aes(x = searches, y = value)) +
+		ggplot2::labs(x = "Number of Searches", y = ylab, title = title) +
+		ggplot2::theme_classic()
+	type = tolower(type)
+	draw_line = grepl("l", type) || grepl("o", type) || grepl("b", type) || grepl("c", type)
+	draw_points = grepl("p", type) || grepl("o", type) || grepl("b", type)
+	if (identical(type, "n")){
+		draw_line = FALSE
+		draw_points = FALSE
+	}
+	if (draw_line && nrow(plot_data) > 1){
+		line_args = list(na.rm = TRUE)
+		if (!is.null(plot_color)){
+			line_args$color = plot_color
+		}
+		if (!is.null(line_size)){
+			line_args$size = line_size
+		}
+		order_plot = order_plot + do.call(ggplot2::geom_line, line_args)
+	}
+	if (draw_points && nrow(plot_data) > 0){
+		point_args = list(na.rm = TRUE)
+		if (!is.null(plot_color)){
+			point_args$color = plot_color
+		}
+		if (!is.null(point_size)){
+			point_args$size = point_size
+		}
+		if (!is.null(point_shape)){
+			point_args$shape = point_shape
+		}
+		order_plot = order_plot + do.call(ggplot2::geom_point, point_args)
+	}
+	if ((nrow(plot_data) == 0) || (!draw_line && !draw_points)){
+		order_plot = order_plot + ggplot2::geom_blank()
+	}
+	if (!is.null(xlim) || !is.null(ylim)){
+		order_plot = order_plot + ggplot2::coord_cartesian(xlim = xlim, ylim = ylim)
+	}
+	list(plot = order_plot, val_order_stats = val_order_stats)	
 }
 
 # Returns the number of vectors found by the greedy design search
@@ -317,64 +488,79 @@ greedySearchCurrentProgress = function(obj){
 #' @param obj 			The \code{greedy_experimental_design} object that is currently running the search
 #' @param max_vectors	The number of design vectors you wish to return. \code{NULL} returns all of them. 
 #' 						This is not recommended as returning over 1,000 vectors is time-intensive. The default is 9. 
-#' @param form			Which form should it be in? The default is \code{one_zero} for 1/0's or \code{pos_one_min_one} for +1/-1's. 
+#' @param form			Which form should it be in? The default is \code{one_zero} for 1/0's or \code{pos_one_min_one} for +1/-1's.
 #' 
 #' @author Adam Kapelner
 #' @examples
-#'  \dontrun{
-#' 	library(MASS)
-#' 	data(Boston)
-#'  #pretend the Boston data was an experiment setting 
-#' 	#first pull out the covariates
-#'  X = Boston[, 1 : 13]
-#'  #begin the greedy design search
-#' 	ged = initGreedyExperimentalDesignObject(X, 
-#' 		max_designs = 1000, num_cores = 2, objective = "abs_sum_diff")
-#' 	#wait
-#' 	res = resultsGreedySearch(ged, max_vectors = 2)
-#' 	design = res$ending_indicTs[, 1] #ordered already by best-->worst
-#'  design
-#'  #what is the balance on this vector?
-#' 	res$obj_vals[1]
-#' 	#compute balance explicitly in R to double check
-#' 	compute_objective_val(X, design) #same as above
-#' 	#how far have we come?
-#' 	ged
-#' 	#we can cut it here
-#' 	stopSearch(ged)
-#' 	}
+#' \dontrun{
+#' set.seed(1)
+#' X = matrix(rnorm(20), nrow = 10)
+#' ged = initGreedyExperimentalDesignObject(
+#'   X,
+#'   max_designs = 5,
+#'   num_cores = 1,
+#'   objective = "abs_sum_diff",
+#'   start = TRUE,
+#'   wait = TRUE,
+#'   verbose = FALSE
+#' )
+#' res = resultsGreedySearch(ged, max_vectors = 2)
+#' res$obj_vals
+#' }
 #' @export
 resultsGreedySearch = function(obj, max_vectors = 9, form = "one_zero"){
+	if (!is.null(max_vectors)){
+		assertCount(max_vectors, positive = TRUE)
+	}
 	obj_vals = .jcall(obj$java_obj, "[D", "getObjectiveVals")
 	num_iters = .jcall(obj$java_obj, "[I", "getNumIters")
 	#these two are in order, so let's order the indicTs by the final objective values
 	ordered_indices = order(obj_vals)
-	last_index = ifelse(is.null(max_vectors), obj$max_designs, min(max_vectors + 1, obj$max_designs))
-	
+	num_completed = length(obj_vals)
+	if (is.null(max_vectors)){
+		last_index = num_completed
+	} else {
+		last_index = min(max_vectors, num_completed)
+	}
 	ending_indicTs = NULL
 	starting_indicTs = NULL
 	switches = NULL
 	xbarj_diffs = NULL
 	obj_val_by_iters = NULL
 	pct_vec_same = NULL
-	ending_indicTs = .jcall(obj$java_obj, "[[I", "getEndingIndicTs", as.integer(ordered_indices[1 : last_index] - 1), simplify = TRUE)
-	if (form == "pos_one_min_one"){
-		ending_indicTs = (ending_indicTs - 0.5) * 2
-	}
-	if (obj$diagnostics){
-		starting_indicTs = .jcall(obj$java_obj, "[[I", "getStartingIndicTs", as.integer(ordered_indices[1 : last_index] - 1), simplify = TRUE)
+	ordered_java_indices = integer(0)
+	ordered_java_indices_j = NULL
+	if (last_index > 0){
+		ordered_java_indices = as.integer(ordered_indices[1 : last_index] - 1)
+		ordered_java_indices_j = .jarray(ordered_java_indices)
+		ending_indicTs = .jcall(obj$java_obj, "[[I", "getEndingIndicTs", ordered_java_indices_j, simplify = TRUE)
 		if (form == "pos_one_min_one"){
-			starting_indicTs = (starting_indicTs - 0.5) * 2
+			if (length(ending_indicTs) > 0 && min(ending_indicTs) >= 0 && max(ending_indicTs) <= 1){
+				ending_indicTs = (ending_indicTs - 0.5) * 2
+			}
+		} else if (form == "one_zero" && length(ending_indicTs) > 0 && min(ending_indicTs) < 0){
+			ending_indicTs = (ending_indicTs + 1) / 2
 		}
-		switches = .jcall(obj$java_obj, "[[[I", "getSwitchedPairs", as.integer(ordered_indices[1 : last_index] - 1), simplify = TRUE)
-		xbarj_diffs = .jcall(obj$java_obj, "[[[D", "getXbarjDiffs", as.integer(ordered_indices[1 : last_index] - 1), simplify = TRUE)
-		obj_val_by_iters = .jcall(obj$java_obj, "[[D", "getObjValByIter", as.integer(ordered_indices[1 : last_index] - 1), simplify = TRUE)
+	}
+	if (obj$diagnostics && last_index > 0){
+		starting_indicTs = .jcall(obj$java_obj, "[[I", "getStartingIndicTs", ordered_java_indices_j, simplify = TRUE)
+		if (form == "pos_one_min_one"){
+			if (length(starting_indicTs) > 0 && min(starting_indicTs) >= 0 && max(starting_indicTs) <= 1){
+				starting_indicTs = (starting_indicTs - 0.5) * 2
+			}
+		} else if (form == "one_zero" && length(starting_indicTs) > 0 && min(starting_indicTs) < 0){
+			starting_indicTs = (starting_indicTs + 1) / 2
+		}
+		switches = .jcall(obj$java_obj, "[[[I", "getSwitchedPairs", ordered_java_indices_j, simplify = TRUE)
+		xbarj_diffs = .jcall(obj$java_obj, "[[[D", "getXbarjDiffs", ordered_java_indices_j, simplify = TRUE)
+		obj_val_by_iters = .jcall(obj$java_obj, "[[D", "getObjValByIter", ordered_java_indices_j, simplify = TRUE)
 		
 		pct_vec_same = colSums(starting_indicTs == ending_indicTs) / length(starting_indicTs[, 1]) * 100
 	}
 	greedy_experimental_design_search_results = list(
 		ordered_indices = ordered_indices,
 		obj_vals = obj_vals[ordered_indices], 
+		obj_vals_orig_order = obj_vals,
 		obj_vals_unordered = obj_vals,
 		num_iters = num_iters[ordered_indices], 
 		orig_order = ordered_indices, 
@@ -407,13 +593,11 @@ resultsGreedySearch = function(obj, max_vectors = 9, form = "one_zero"){
 #' 
 #' @author Adam Kapelner
 #' @examples
-#'  \dontrun{
-#' 	optimize_asymmetric_treatment_assignment(n = 100) 
-#'  #nT = nC = 50
-#' 	optimize_asymmetric_treatment_assignment(n = 100, c_treatment = 2, c_control = 1) 
-#'  #nT = 66, nC = 34
-#' 	optimize_asymmetric_treatment_assignment(c_total_max = 50, c_treatment = 2, c_control = 1) 
-#' 	}
+#' \dontrun{
+#' optimize_asymmetric_treatment_assignment(n = 100)
+#' optimize_asymmetric_treatment_assignment(n = 100, c_treatment = 2, c_control = 1)
+#' optimize_asymmetric_treatment_assignment(c_total_max = 50, c_treatment = 2, c_control = 1)
+#' }
 #' @export
 optimize_asymmetric_treatment_assignment = function(
 		c_treatment = NULL,
@@ -431,12 +615,7 @@ optimize_asymmetric_treatment_assignment = function(
 		} else {
 			checkNumeric(c_treatment, lower = .Machine$double.xmin, finite = TRUE)
 			checkNumeric(c_control,   lower = .Machine$double.xmin, finite = TRUE)
-			c_const = c_treatment / c_control
-			nT = if (c_treatment > c_control){
-				 	ceiling(n / (1 + sqrt(c_const)))
-		         } else {
-				 	floor(n / (1 + sqrt(c_const)))
-		         }
+			nT = ceiling(n * c_treatment / (c_treatment + c_control))
 		 	nC = n - nT
 		 	c_total = nT * c_treatment + nC * c_control
 		 	list(n = n, nT = nT, nC = nC, c_treatment = c_treatment, c_control = c_control, c_total = c_total)
@@ -459,5 +638,3 @@ optimize_asymmetric_treatment_assignment = function(
 		list(n = nT + nC, nT = nT, nC = nC, c_total = c_total, c_total_max = c_total_max, c_treatment = c_treatment, c_control = c_control)
 	}
 }
-
-
