@@ -12,18 +12,45 @@
 #' @return				An matrix of dimension $R$ x $n$ where $R$ is the number of Hadamard allocations.
 #' 
 #' @author Adam Kapelner
+#' @examples
+#' \dontrun{
+#' set.seed(1)
+#' X = matrix(rnorm(16), nrow = 8)
+#' W = hadamardExperimentalDesign(X, strict = TRUE, form = "one_zero")
+#' dim(W)
+#' }
 #' @export
-hadamardExperimentalDesign = function(X, strict = TRUE, form = "zero_one"){
+hadamardExperimentalDesign = function(X, strict = TRUE, form = "one_zero"){
 	assertMatrixWithEvenNumRows(X)
 	n = nrow(X)
-	W = t(survey::hadamard(n))[-1, ] #the first row is always just 1's, so remove it
-	#also retain only those that are forced balance (I think this is all of them, but I want to just make sure
-	W = W[rowMeans(W) == 0.5, ]
-	if (strict){
-		if (ncol(W) != n){
-			stop("A Hadamard matrix for this sample size doesn't exist. Use strict = FALSE to approximate one randomly.")
-		}		
+	is_power_of_two = function(x){
+		x > 0 && bitwAnd(x, x - 1) == 0
+	}
+	build_hadamard = function(m){
+		if (m == 1){
+			return(matrix(1, 1, 1))
+		}
+		if (!is_power_of_two(m)){
+			stop("Hadamard order must be a power of two.")
+		}
+		H = build_hadamard(m / 2)
+		rbind(cbind(H, H), cbind(H, -H))
+	}
+	if (is_power_of_two(n)){
+		m = n
+	} else if (strict){
+		stop("A Hadamard matrix for this sample size doesn't exist. Use strict = FALSE to approximate one randomly.")
 	} else {
+		m = 2 ^ ceiling(log2(n))
+	}
+	W = (build_hadamard(m) + 1) / 2
+	W = t(W)[-1, , drop = FALSE] #the first row is always just 1's, so remove it
+	#also retain only those that are forced balance (I think this is all of them, but I want to just make sure
+	W = W[rowMeans(W) == 0.5, , drop = FALSE]
+	if (ncol(W) != n){
+		if (strict){
+			stop("A Hadamard matrix for this sample size doesn't exist. Use strict = FALSE to approximate one randomly.")
+		}
 		Wcut = matrix(NA, nrow = nrow(W), ncol = n)
 		for (r in 1 : nrow(W)){
 			w = W[r, ]
@@ -51,5 +78,3 @@ assertMatrixWithEvenNumRows = function(x, .var.name = vname(x), add = NULL) {
 }
 
 
-
-
